<?php
	/*

	Plugin Name:    Photos Flickr
	Plugin URI:     http://dannyman.toldme.com/photos-flickr/

	Author:         Danny Howard
	Author URI:     http://dannyman.toldme.com/

	Description:    Photos Flickr is an early version of a plugin that I am developing for WordPress blog software to display a user's Flickr album within their blog.  This plugin is far from fully-baked.  It does some of what I want, so far, and there is more work to do.

	Version:        0.5

	*/

	// require_once("photos-flickr-conf.php");
	require_once("phpFlickr/phpFlickr.php");

	// set by init()
	$f;                 // phpFlickr object -- API handle
    $person;            // findByUsername is supposed to return NSID, but ...
    $nsid;              // NSID (UID) on Flickr
    $photos_base_url;   // So we can get self-referential

    // other globals
    $error_message;     // In case we want to flash the user / admin

    // CONFIGURATION -- From photos-flickr-conf.php
	/* pretty obvious */
	$flickr_user    =   get_option('flickr_user');
	/* Get one at http://www.flickr.com/services/api/key.gne */
	$flickr_API_key =   'efc050ee72d23be84595fcf05995ebf6';

	/* Enable database-backed cache.  These defaults should "just work"
	* but if things get broken, try commenting out the following lines.
    *
    * TODO: Configurability-via-admin?
	* */
	$flickr_cache_db    = "mysql://".DB_USER.":".DB_PASSWORD."@".DB_HOST."/".DB_NAME;
	$flickr_cache_exp   = '600';
	$flickr_cache_table = $GLOBALS['table_prefix']."flickr_cache";

    // well, if we have to . . .
    // $debug = True;

    /*
    photos_init -- MANDATORY call once before you call other stuff:
    - Connects to Flickr, setting up phpFlickr handle ($f)
    - Connects to cache database
    - Sets up $person and $nsid for our convenience
    - Sets up $photos_base_url appropriately

    TODO: If we are hooking in to WordPress' init() then this should
    check to see if we're actually doing any photos before it goes
    pinging Flickr and setting up a database connection, huh?

    */
	function photos_init() {
        global $f, $person, $nsid, $photos_base_url;
        global $flickr_user, $flickr_API_key;
        global $flickr_cache_db, $flickr_cache_exp, $flickr_cache_table;
        global $init_done; // Careful not to overdo it.
        // Connects to Flickr, setting up phpFlickr handle ($f)
		photos_init_phpFlickr();
		// Sets up $person and $nsid for our convenience
		// $person = $f->people_findByUsername($GLOBALS['flickr_user']);
		$person = get_flickr_user($f, $flickr_user);
        // WTF: findByUsername used to return NSID directly.
		$nsid = $person['id'];
        // Sets up $photos_base_url appropriately
        // TODO: This should be configurable?
        $photos_base_url = get_permalink();
        // /?page_id=3 -> /?page_id=3&  . . . /photos/ -> /photos/?
        if( strstr($photos_base_url, '?') ) { $photos_base_url .= '&'; }
        else { $photos_base_url .= '?'; }
        if( $debug ) { echo "photos_base_url: $photos_base_url<br />"; }
	}
    // wp_head is far anough to build $photos_base_url
    // BUT this is somewhat silly.
    // add_action('wp_head', 'photos_init');

    // photos_init_phpFlickr -- connects to Flickr, setting up $f
    function photos_init_phpFlickr() {
        global $f, $flickr_API_key;
        global $flickr_cache_db, $flickr_cache_exp, $flickr_cache_table;
        if( $f ) { return; } // Why reinvent the $f?
        $f = new phpFlickr($flickr_API_key);
        // Connects to cache database
		if( $flickr_cache_db ) {
			$f->enableCache('db', $flickr_cache_db,
				$flickr_cache_exp, $flickr_cache_table);
		}
    }

    /*
    photos_config_page -- Hooks into WordPress Admin menu to manage
    options and whatnot.
    */
    function photos_config_page() {
        global $flickr_user, $flickr_API_key;
        global $flickr_cache_db, $flickr_cache_exp, $flickr_cache_table;

        // Add submenu to Plugins tab: Photos Flickr
        add_submenu_page('plugins.php',
            'Photos Flickr', 'Photos Flickr', 'manage_options',
            'photos-flickr-config', 'photos_flickr_config_page');
    }
    add_action('admin_menu', 'photos_config_page');

    /*
    get_flickr_user -- private function to map NSID, username, or e-mail
    to $flickr_user, which will be an NSID.

    Meant to be called by init() as a convenience and also by the Admin
    panel as a sanity-check.

    Returns a $person ... I think.
    */
    function get_flickr_user( $f, $get_flickr_user ) {
        global $error_message;
        // DEFAULT: admin_email
        if( !$get_flickr_user ) {
            $get_flickr_user = get_option('admin_email');
        }
        if( !$get_flickr_user ) {
            $error_message .= "get_flickr_user: I don't think this ".
            "request is true: '$get_flickr_user'!";
            return(False);
        }
        if( $debug ) { echo "Looking for get_flickr_user == $get_flickr_user<br />"; }
        // CASE 1: Valid NSID
        $get_person = $f->people_getInfo($get_flickr_user);
        if( isset($get_person['id']) ) { 
            if( $debug ) { echo "get_flickr_user--NSID: ".$get_person['id']."<br />"; }
            return($get_person);
        }
        // CASE 2: Valid username
        $get_person = $f->people_findByUsername($get_flickr_user);
        if( isset($get_person['id']) ) { 
            if( $debug ) { echo "get_flickr_user--Username: ".$get_person['id']."<br />"; }
            return($get_person);
        }
        // CASE 3: Valid e-mail
        $get_person = $f->people_findByEmail($get_flickr_user);
        if( isset($get_person['id']) ) { 
            if( $debug ) { echo "get_flickr_user--Email: ".$get_person['id']."<br />"; }
            return($get_person);
        }
        // CASE 4: Nyet!
        $error_message .= "get_flickr_user: I tried to search by ".
            "NSID, username, and e-mail, but found nothing for ".
            "'$get_flickr_user'!";
        return(False);
    }

    /*
    photos_flickr_config_page -- Displays the actual administrative
    configuration page.  Byagh!
    */
    function photos_flickr_config_page() {
    /* WTF: Stolen heavily from the Akismet plugin, which lacks
     * comments, so who knows what the f_ck I'm doing wrong here? */
        global $flickr_API_key, $flickr_user;
        global $f;
        global $error_message;

    // STEP 2: Parse form submissions
    // TODO: Do we need to check user authentication level!?
    if ( isset($_POST['submit']) ) {
        photos_init_phpFlickr();
        if( isset($_POST['flickr_API_key']) ) {
            $flickr_API_key = $_POST['flickr_API_key'];
            update_option('flickr_API_key', $flickr_API_key);
        }
        if( isset($_POST['flickr_user']) ) {
            $person = get_flickr_user( $f, $_POST['flickr_user'] );
            if( $person ) {
                $flickr_user = $person['id'];
                update_option('flickr_user', $flickr_user);
            }
        }
    }

    // STEP 1: Build a form ...
?>
<?php if ( !empty($_POST)  && empty($error_message) ) : ?>
<div id="message" class="updated fade"><p><strong><?php _e('Options saved.') ?></strong></p></div>
<?php elseif ( !empty($_POST) && !empty($error_message) ) : ?>
<div id="message" class="error"><p><strong><?php _e($error_message) ?></strong></p></div>
<?php endif; ?>
<div class="wrap">
    <h2>Photos Flickr</h2>
    <form action="" method="post" id="photos-flickr-config">
        <table class="optiontable">
            <tr valign="top">
                <th scope="row">Flickr User ID:</th>
                <td><input name="flickr_user" type="text"
                value="<?php echo "$flickr_user" ?>" />
                <br />Enter your NSID, Flickr screen name, or the e-mail
                address you used to register with Flickr.  Once found,
                this value will be converted to your NSID.  If you don't
                enter anything, we will try your
                <code>admin_email</code>.</td>
            </tr>
            <tr valign="top">
                <th scope="row">Flickr API Key:</th>
                <td><input name="flickr_API_key" type="text" size="32"
                class="code" value="<?php echo "$flickr_API_key" ?>" />
                <br /><b>Required!</b>  You will need to <a
                href="http://www.flickr.com/services/api/keys/apply/">request
                an API key from flickr.com</a> for this plugin to
                work.</td>
            </tr>
        </table>
    <p class="submit"><input type="submit" name="submit" value="Update Options &raquo;" /></p>
    </form>
</div>
<?php
;
    }

	// "Hello World" function -- useful for testing
	function photos_flickr_hello() {
		// Bring in our flickr object and NSID
		$f = $GLOBALS['f'];
		$nsid = $GLOBALS['nsid'];
        if( $debug ) { print "nsid: $nsid<br />\n"; }
	
	    // Get the friendly URL of the user's photos
	    $photos_url = $f->urls_getUserPhotos($nsid);
        if( $debug ) { print "photos_url: $photos_url<br />\n"; }
   
	    // Get the user's first 36 public photos
		$photos = $f->people_getPublicPhotos($nsid, NULL, 36);
   
	    // Loop through the photos and output the html
	    foreach ((array)$photos['photo'] as $photo) {
			print "<a href=$photos_url$photo[id]>";
	        print "<img border='0' alt='$photo[title]' ".
				"src=" . $f->buildPhotoURL($photo, "Square") . ">";
	        print "</a>";
			$i++;
			// If it reaches the sixth photo, insert a line break
			if ($i % 6 == 0) {
				echo "<br>\n";
	     	}
		}
	}

	// Print a reasonable index page.
	function photos_index(
		$num_per_page = 25, $num_per_row = 5, $page = NULL,
		$thumb_type='square', $thumb_before = '', $thumb_after = '',
		$row_sep = '<br />' ) {
		// Bring in our flickr object and NSID
		$f = $GLOBALS['f'];
		$nsid = $GLOBALS['nsid'];
        $photos_base_url = $GLOBALS['photos_base_url'];
		// IF no page is passed, check $_REQUEST, else set to 1
		if( $page == NULL ) {
			if( $_REQUEST['page'] ) { $page = $_REQUEST['page']; }
			else                    { $page = 1; }
		}

		if( $_REQUEST['tag'] ) {
			$photos = $f->photos_search(array("user_id"=>$nsid,
				"tags"=>$_REQUEST['tag'], "per_page"=>$num_per_page,
				"page"=>$page));
		} else {
			$photos = $f->people_getPublicPhotos($nsid, NULL,
				$num_per_page, $page);
		}

		foreach ((array)$photos['photo'] as $photo) {
			print $thumb_before .
				"<a href=\"".$photos_base_url."photo=$photo[id]\">" .
				"<img class=\"photosThumb\" id=\"photosId_$photo[id]\"" .
				" alt=\"$photo[title]\" title=\"$photo[title]\"" .
				" src=\"".$f->buildPhotoURL($photo, $thumb_type)."\" />" .
				"</a>" .
				$thumb_after."\n";
			$i++;
			if( $i%$num_per_row == 0 ) {
				print $row_sep."\n";
			}
		}
	}

	// Print "paginate" link bar thingy
	function photos_pageindex( $context = 2, $num_per_page = 25,
		$item_sep = ', ', $dotdotdot = " . . . ",$page = NULL ) {

		// Bring in our flickr object and NSID
		$f = $GLOBALS['f'];
		$nsid = $GLOBALS['nsid'];
        $photos_base_url = $GLOBALS['photos_base_url'];
		// IF no page is passed, check $_REQUEST, else set to 1
		if( $page == NULL ) {
			if( $_REQUEST['page'] ) { $page = $_REQUEST['page']; }
			else                    { $page = 1; }
		}

		// Count the number of pages ...
		if( $_REQUEST['tag'] ) {
			$photos = $f->photos_search(array("user_id"=>$nsid,
				"tags"=>$_REQUEST['tag'], "per_page"=>$num_per_page,
				"page"=>$page));
		} else {
			$photos = $f->people_getPublicPhotos($nsid, NULL,
				$num_per_page, $page);
		}
		$pagecount = $photos['pages'];

		$n = 1; $emit = 0; $printme = '';
		while( $n <= $pagecount ) {

			if( // 1, 2, 3 ...
				(1 <= $n && $n <= (1+$context))                    ||
				// ... 7, 8, [9], 10, 11 ...
				(($page-$context) <= $n && $n <= ($page+$context)) ||
				// ... 98, 99, 100
				(($pagecount-$context) <= $n && $n <= $pagecount)  ) {

				if( $emit != 0 ) { print $item_sep; $emit = 0; }
					
				if( $n == $page ) {
					print "<b>$n</b>";
				} else {
					if( $_REQUEST['tag'] ) {
						print "<a href=\"".$photos_base_url.
							"tag=".$_REQUEST['tag'].
							"&page=$n\">$n</a>";
					} else {
						print "<a href=\"".$photos_base_url.
							"page=$n\">$n</a>";
					}
				}
					
				$emit++;
			} else if( $emit != 0 ) { print $dotdotdot; $emit = 0; }

			$n++;
		}

	}

	// Print linked tags for a photo
	function photos_photo_tags( $tag_before = '', $tag_after = ' ',
		$photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];
        $photos_base_url = $GLOBALS['photos_base_url'];

		$photo = $f->photos_getInfo($photo_id);

		if( $photo['tags'] ) {
			foreach( $photo['tags']['tag'] as $tag ) {
				print "$tag_before<a href=\"".$photos_base_url.
					"tag=$tag[_content]\">$tag[raw]</a>$tag_after";
			}
		}
	}

	// Print photo information - photopage URL
	function photos_photo_page_url( $photo_id = NULL ) {
        global $person;
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		print "http://www.flickr.com/photos/".
			$person['username']."/".$photo_id."/";
	}

	// Print photo information - URL
	function photos_photo_url( $size='medium', $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$photo = $f->photos_getInfo($photo_id);
		print $f->buildPhotoURL($photo, $size);
	}

	// Print photo information - description
	function photos_photo_description( $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$photo = $f->photos_getInfo($photo_id);
		$description = str_replace("\n", '<br />', $photo[description]);
		print $description;
	}

	// Print photo information - title
	function photos_photo_title( $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$photo = $f->photos_getInfo($photo_id);
		print $photo[title];
	}

	// Print next photo thumbnail
	function photos_photo_next_url( $thumb_type = 'square',
		$photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$nl = $f->photos_getContext($photo_id);
		if( $nl[nextphoto][id] != 0 ) {
			$photo = $f->photos_getInfo($nl[nextphoto][id]);
			print "<img class=\"photosThumb\" id=\"photosId_".
				$nl[nextphoto][id]."\" src=\"".
				$f->buildPhotoURL($photo, $thumb_type).
				"\" title=\"".$nl[nextphoto][title]."\" />";
		}
	}
	// Print prev photo thumbnail
	function photos_photo_prev_url( $thumb_type = 'square',
		$photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$nl = $f->photos_getContext($photo_id);
		if( $nl[prevphoto][id] != 0 ) {
			$photo = $f->photos_getInfo($nl[prevphoto][id]);
			print "<img class=\"photosThumb\" id=\"photosId_".
				$nl[prevphoto][id]."\" src=\"".
				$f->buildPhotoURL($photo, $thumb_type).
				"\" title=\"".$nl[prevphoto][title]."\" />";
		}
	}

	// Print next photo href
	function photos_photo_next_href( $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];
        $photos_base_url = $GLOBALS['photos_base_url'];

		$nl = $f->photos_getContext($photo_id);
		if( $nl[nextphoto][id] != 0 ) {
			print $photos_base_url."photo=".$nl[nextphoto][id];
		}
	}
	// Print prev photo href
	function photos_photo_prev_href( $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];
        $photos_base_url = $GLOBALS['photos_base_url'];

		$nl = $f->photos_getContext($photo_id);
		if( $nl[prevphoto][id] != 0 ) {
			print $photos_base_url."photo=".$nl[prevphoto][id];
		}
	}

	// Print next photo title
	function photos_photo_next_title( $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$nl = $f->photos_getContext($photo_id);
		if( $nl[prevphoto][id] != 0 ) {
			print $nl[nextphoto][title];
		}
	}
	// Print prev photo title
	function photos_photo_prev_title( $photo_id = NULL ) {
		if( $photo_id == NULL ) { $photo_id = $_REQUEST['photo']; }
		// Bring in our flickr object
		$f = $GLOBALS['f'];

		$nl = $f->photos_getContext($photo_id);
		if( $nl[prevphoto][id] != 0 ) {
			print $nl[prevphoto][title];
		}
	}

	// Are we a page for one photo, or some nav index?
	function is_photo() {
		if( $_REQUEST['photo'] ) { return $_REQUEST['photo']; } else { return false; }
	}

?>
